/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors.bo;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.log4j.Logger;

public final class GenericItemUtil {

	private static final String GENERIC_ITEMS = "GenericItems";
	private static final String GENERIC_ITEM = "GenericItem";
	private static final String XML_FILE_NAME = "genericItem.xml";
	
	private static final Logger LOGGER = Logger.getLogger(GenericItemUtil.class);

	private GenericItemUtil() {
	};

	public static SourceBean genericItemListToSourceBean(
			final List<GenericItemInterface> genericItemList) {
		SourceBean genericItemsSB = null;
		try {
			genericItemsSB = new SourceBean(GENERIC_ITEMS);
			for (GenericItemInterface genericItem : genericItemList) {
				Set<String> keys = genericItem.getKeys();
				SourceBean genericItemSB = new SourceBean(GENERIC_ITEM);
				for (String key : keys) {
					SourceBean keySB = new SourceBean(key);
					keySB.setCharacters(genericItem.getValue(key));
					genericItemSB.setAttribute(keySB);
				}
				genericItemsSB.setAttribute(genericItemSB);
			}
		} catch (SourceBeanException e) {
			LOGGER.debug(e.getLocalizedMessage());
		}
		return genericItemsSB;
	}

	public static String sourceBeanToZipFile(final String archiveDir,
			final String zipFileName, final SourceBean sourceBean) {
		String toReturn = null;
		try {
			File tempDirectory = new File(archiveDir);
			tempDirectory.mkdir();
			File archive = new File(tempDirectory, zipFileName);
			toReturn = archive.getAbsolutePath();
			ZipOutputStream out = new ZipOutputStream(new BufferedOutputStream(
					new FileOutputStream(archive)));
			out.putNextEntry(new ZipEntry(XML_FILE_NAME));
			byte[] sourceBeanByte = sourceBean.toString().getBytes();
			out.write(sourceBeanByte);
			out.flush();
			out.close();
		} catch (IOException e) {
			LOGGER.debug(e.getLocalizedMessage());
		}
		return toReturn;
	}

}
