/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors.tablehandlers;

import it.eng.spago.base.SourceBean;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.extractors.bo.InterfaceItem;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.List;

public abstract class AbstractTableHandler implements TableHandlerInterface {

	protected void queryExecutor(final DataConnection dataConnection,
			final String query) throws EMFUserError {
		SQLCommand sqlCommand = null;
		DataResult dataResult = null;
		try {
			sqlCommand = dataConnection.createInsertCommand(query);
			sqlCommand.execute();
		} catch (EMFInternalError e) {
			throw new EMFUserError(e.getCategory(), e.getDescription());
		} finally {
			Utils.releaseResources(dataConnection, sqlCommand, dataResult);
		}
	}

	public String transformTableName(final String tableName) {
		return "FT_" + tableName.trim().replace(" ", "").toUpperCase();
	}

	protected SourceBean selectQueryExecutor(
			final DataConnection dataConnection, final String query)
			throws EMFUserError {
		SourceBean result = null;
		SQLCommand sqlCommand = null;
		DataResult dataResult = null;
		try {
			sqlCommand = dataConnection.createSelectCommand(query);
			dataResult = sqlCommand.execute();
			ScrollableDataResult scrollableDataResult = (ScrollableDataResult) dataResult
					.getDataObject();
			result = scrollableDataResult.getSourceBean();
		} catch (EMFInternalError e) {
			throw new EMFUserError(e.getCategory(), e.getDescription());
		} finally {
			Utils.releaseResources(dataConnection, sqlCommand, dataResult);
		}
		return result;
	}

	public boolean hasThisTable(final Connection connection,
			final String tableName) throws EMFUserError {
		boolean toReturn = false;
		DatabaseMetaData dbmd = null;
		try {
			dbmd = connection.getMetaData();
			// Specify the type of object; in this case we want tables
			String[] types = { "TABLE" };
			ResultSet resultSet = dbmd.getTables(null, null, "%", types);

			// Get the table names
			while (resultSet.next()) {
				// Get the table name
				String dbTableName = resultSet.getString(3);
				if (dbTableName.toUpperCase().equals(tableName)) {
					toReturn = true;
					break;
				}
			}
		} catch (SQLException e) {
			throw new EMFUserError(EMFErrorSeverity.ERROR, 100);

		}
		return toReturn;
	}

	public void dropFactTable(final DataConnection dataConnection,
			final String tableName) throws EMFUserError {
		String query = getSqlCodeGenerator().generateDropTableQuery(tableName);
		queryExecutor(dataConnection, query);
	}

	public void createFactTable(final DataConnection dataConnection,
			final String tableName, final List interfaceFieldList)
			throws EMFUserError {
		String query = getSqlCodeGenerator().generateCreateTableQuery(
				tableName, interfaceFieldList);
		queryExecutor(dataConnection, query);
	}

	public void updateFactTable(final Date executionDate,
			final DataConnection dataConnection, final String tableName,
			final InterfaceItem interfaceItem) throws EMFUserError {
		String updateQuery = getSqlCodeGenerator().generateUpdateQuery(
				executionDate, tableName, interfaceItem);
		queryExecutor(dataConnection, updateQuery);
	}

	public void insertIntoFactTable(final Date executionDate,
			final DataConnection dataConnection, final String tableName,
			final InterfaceItem interfaceItem, final Integer logId)
			throws EMFUserError {
		String insertQuery = getSqlCodeGenerator().generateInsertQuery(
				executionDate, tableName, interfaceItem, logId);
		queryExecutor(dataConnection, insertQuery);
	}

	public SourceBean selectFromFactTable(final DataConnection dataConnection,
			final String tableName, final InterfaceItem interfaceItem,
			final Date operationDate) throws EMFUserError {
		String selectQuery = getSqlCodeGenerator().generateSelectQuery(
				tableName, interfaceItem, operationDate);
		SourceBean selectResult = selectQueryExecutor(dataConnection,
				selectQuery);
		return selectResult;
	}

	public abstract SqlCodeGeneratorInterface getSqlCodeGenerator();

}
