/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors.xml;

import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.extractors.AbstractExtractor;
import it.eng.spago4q.extractors.GenericItemSetterInterface;
import it.eng.spago4q.extractors.bo.GenericItemInterface;

import java.util.ArrayList;
import java.util.List;

import org.dom4j.DocumentException;
import org.dom4j.io.SAXReader;

public class SaxXmlExtractor extends AbstractExtractor implements GenericItemSetterInterface{
	
	private List<GenericItemInterface> result = new ArrayList<GenericItemInterface>();
	private static final String FILE = "FILE";
	private static final String XPATH = "XPATH";

	private String xPathExpression;
	private String document;

	public void setGenericItem(final GenericItemInterface genericItem){
		result.add(genericItem);
	}

	protected void generateResult() throws EMFUserError {
		SAXReader reader = new SAXReader();
		reader.addHandler(getXPathExpression(), new SaxElementHandler(this));
		try {
			reader.read(getDocument());
		} catch (DocumentException e) {
			throw new EMFUserError(EMFErrorSeverity.ERROR, e.getMessage());
		}
	}

	private String getDocument() throws EMFUserError {
		if (document == null) {
			throw new EMFUserError(EMFErrorSeverity.ERROR, "Impossible to create the document.");
		}
		return document;
	}

	private String getXPathExpression() throws EMFUserError {
		if (xPathExpression == null) {
			throw new EMFUserError(EMFErrorSeverity.ERROR, "Impossible to create the XPath expression.");
		}
		return xPathExpression;
	}

	@Override
	protected void setUp() {
		setDocument(readDataSourceParameterValue(FILE));
		setXPathExpression(readOperationParameterValue(XPATH));
	}

	protected void setXPathExpression(final String xPathExp) {
		xPathExpression = xPathExp;
	}

	protected void setDocument(final String doc) {
		document = doc;
	}

	@Override
	protected void tearDown() {
	}

	@Override
	protected List<GenericItemInterface> extract() throws EMFUserError {
		generateResult();
		return result;
	}
}
