package it.eng.spago4q.extractors;

import java.util.Iterator;
import java.util.List;

import it.eng.spago4q.bo.EOperation;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QFKDAO;
import it.eng.spago4q.extractors.AbstractExtractor;
import it.eng.spago4q.extractors.xml.SaxXmlExtractor;
import it.eng.spago4q.testUtils.AbstractDBTestCase;

public class AbstractExtractorTest extends AbstractDBTestCase {

	private static final String FLAT_FILE = "test/flatFile/AbstractExtractorTestFlatFile.xml";

	private AbstractExtractor extractor;

	public AbstractExtractorTest(final String value) {
		super(value);
	}

	@Override
	protected String getFlatFile() {
		return FLAT_FILE;
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		try {
			// get the operation from the DB
			IES4QFKDAO operationDAO = (IES4QFKDAO) DAOFactory
					.getDAO("EOperationDAO");
			EOperation operation = (EOperation) operationDAO.loadObjectById(1);

			// Create the extractor
			extractor = new SaxXmlExtractor();
			extractor.init(operation);
		} catch (Exception e) {
			fail();
		}
	}

	public void testReadDataSourceParameterValueList() {
		// Check the number of Data Source parameters
		assertEquals(2, extractor.getDataSourceParameter().size());
		
		List<String> dsPars = extractor.readDataSourceParameterValueList("DSPAR");

		assertEquals(2, dsPars.size());

		Integer value = 1;
		for (Iterator iterator = dsPars.iterator(); iterator.hasNext();) {
			assertEquals(value.toString(),(String) iterator.next());
			value ++;
		}
	}

	public void testReadOperationParameterValueList() {
		assertEquals(3, extractor.getOperationParameter().size());

		List<String> oPars = extractor.readOperationParameterValueList("OPAR"); 
		assertEquals(3, oPars.size());
		
		Character value = 'a';
		for (Iterator iterator = oPars.iterator(); iterator.hasNext();) {
			assertEquals(value.toString(),(String) iterator.next());
			value ++;
		}
	}
}
