package it.eng.spago4q.importer;

import it.eng.spago4q.exporter.InterfaceTypeExporter;
import it.eng.spago4q.metadata.S4qEDomainValue;
import it.eng.spago4q.metadata.S4qEInterfaceField;
import it.eng.spago4q.metadata.S4qEInterfaceType;
import it.eng.spago4q.testUtils.AbstractDBTestCase;

import java.io.File;
import java.io.FileOutputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.HashSet;
import java.util.Set;

import org.dbunit.Assertion;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.database.DatabaseConnection;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.ITable;
import org.dbunit.dataset.xml.FlatXmlDataSet;
import org.dbunit.ext.hsqldb.HsqldbDataTypeFactory;

public class EInterfaceTypeImportHibImplTest extends AbstractDBTestCase {
	
	private static final String FLAT_FILE = "test/flatFile/s4qFlatFile.xml";

	public EInterfaceTypeImportHibImplTest(final String value) {
		super(value);
	}

	@Override
	protected String getFlatFile() {
		return FLAT_FILE;
	}
	
	public void test1ImportDataOWTrue() {
		try{
		S4qEInterfaceType interfaceType = new S4qEInterfaceType();
		interfaceType.setIdInterfaceType(23);
		interfaceType.setName("Test Interface");
		interfaceType.setDescription("Description Interface Type");
		interfaceType.setTablename("FACT_TABLE_NAME");

		S4qEInterfaceField interfaceField = new S4qEInterfaceField();
		interfaceField.setName("interface Field Name");
		interfaceField.setKeyField(true);
		interfaceField.setSensible(false);
		interfaceField.setFieldType("STRING");
		
		S4qEDomainValue domainvalue1 = new S4qEDomainValue();
		domainvalue1.setValue("value 1");
		
		S4qEDomainValue domainvalue2 = new S4qEDomainValue();
		domainvalue2.setValue("value 2");
		
		Set s4qEDomainValues = new HashSet(0);
		
		s4qEDomainValues.add(domainvalue1);
		s4qEDomainValues.add(domainvalue2);
		interfaceField.setS4qEDomainValues(s4qEDomainValues);

		S4qEInterfaceField interfaceField2 = new S4qEInterfaceField();
		interfaceField2.setName("interface Field Name 2");
		interfaceField2.setKeyField(false);
		interfaceField2.setSensible(true);
		interfaceField2.setFieldType("Double");
		
		Set s4qEInterfaceFields = new HashSet(0);
		
		s4qEInterfaceFields.add(interfaceField);
		s4qEInterfaceFields.add(interfaceField2);
		
		interfaceType.setS4qEInterfaceFields(s4qEInterfaceFields);
		
		InterfaceTypeExporter interfaceTypeExporter = new InterfaceTypeExporter();

		String xml = interfaceTypeExporter.exportInterfaceType(interfaceType); 
		System.out.println(xml);
		
		S4qEInterfaceType s4qInterfaceTypeFromInput = interfaceTypeExporter.importInterfaceType(xml);
		
		EInterfaceTypeImport interfaceTypeImport = new EInterfaceTypeImportHibImpl();
		
		interfaceTypeImport.importData(s4qInterfaceTypeFromInput, 1, true);

		assertInterfaceTypeTables("test/flatFile/InterfaceType/Test1OWTrue.xml");
		}catch(Exception e){
			fail();
		}
	}
	
	//public void test2ImportDataOWFalse() {
//		S4qEInterfaceType interfaceType = new S4qEInterfaceType();
//		interfaceType.setIdInterfaceType(23);
//		interfaceType.setName("Test Interface");
//		interfaceType.setDescription("Description Interface Type");
//		interfaceType.setTablename("FACT_TABLE_NAME");
//
//		S4qEInterfaceField interfaceField = new S4qEInterfaceField();
//		interfaceField.setName("interface Field Name");
//		interfaceField.setKeyField(true);
//		interfaceField.setSensible(false);
//		interfaceField.setFieldType("STRING");
//		
//		S4qEDomainValue domainvalue1 = new S4qEDomainValue();
//		domainvalue1.setValue("value 1");
//		
//		S4qEDomainValue domainvalue2 = new S4qEDomainValue();
//		domainvalue2.setValue("value 2");
//		
//		Set s4qEDomainValues = new HashSet(0);
//		
//		s4qEDomainValues.add(domainvalue1);
//		s4qEDomainValues.add(domainvalue2);
//		interfaceField.setS4qEDomainValues(s4qEDomainValues);
//
//		S4qEInterfaceField interfaceField2 = new S4qEInterfaceField();
//		interfaceField2.setName("interface Field Name 2");
//		interfaceField2.setKeyField(false);
//		interfaceField2.setSensible(true);
//		interfaceField2.setFieldType("Double");
//		
//		Set s4qEInterfaceFields = new HashSet(0);
//		
//		s4qEInterfaceFields.add(interfaceField);
//		s4qEInterfaceFields.add(interfaceField2);
//		
//		interfaceType.setS4qEInterfaceFields(s4qEInterfaceFields);
//		
//		InterfaceTypeExporter interfaceTypeExporter = new InterfaceTypeExporter();
//
//		String xml = interfaceTypeExporter.exportInterfaceType(interfaceType); 
//		System.out.println(xml);
//		
//		S4qEInterfaceType s4qInterfaceTypeFromInput = interfaceTypeExporter.importInterfaceType(xml);
//		
//		EInterfaceTypeImport interfaceTypeImport = new EInterfaceTypeImportHibImpl();
//		
//		interfaceTypeImport.importData(s4qInterfaceTypeFromInput, false);
//		
//		
//
		//assertInterfaceTypeTables("test/flatFile/InterfaceType/Test1OWTrue.xml");
		
//	}
	
	
//	public void test2ImportDataOWTrue() {
//		S4qEInterfaceType interfaceType = new S4qEInterfaceType();
//		interfaceType.setIdInterfaceType(23);
//		interfaceType.setName("Test Interface");
//		interfaceType.setDescription("Description Interface Type");
//		interfaceType.setTablename("FACT_TABLE_NAME");
//
//		S4qEInterfaceField interfaceField = new S4qEInterfaceField();
//		interfaceField.setName("interface Field Name");
//		interfaceField.setKeyField(true);
//		interfaceField.setSensible(false);
//		interfaceField.setFieldType("STRING");
//		
//		S4qEDomainValue domainvalue1 = new S4qEDomainValue();
//		domainvalue1.setValue("value 1");
//		
//		S4qEDomainValue domainvalue2 = new S4qEDomainValue();
//		domainvalue2.setValue("value 2");
//		
//		Set s4qEDomainValues = new HashSet(0);
//		
//		s4qEDomainValues.add(domainvalue1);
//		s4qEDomainValues.add(domainvalue2);
//		interfaceField.setS4qEDomainValues(s4qEDomainValues);
//
//		S4qEInterfaceField interfaceField2 = new S4qEInterfaceField();
//		interfaceField2.setName("interface Field Name 2");
//		interfaceField2.setKeyField(false);
//		interfaceField2.setSensible(true);
//		interfaceField2.setFieldType("Double");
//		
//		Set s4qEInterfaceFields = new HashSet(0);
//		
//		s4qEInterfaceFields.add(interfaceField);
//		s4qEInterfaceFields.add(interfaceField2);
//		
//		interfaceType.setS4qEInterfaceFields(s4qEInterfaceFields);
//		
//		InterfaceTypeExporter interfaceTypeExporter = new InterfaceTypeExporter();
//
//		String xml = interfaceTypeExporter.exportInterfaceType(interfaceType); 
//		System.out.println(xml);
//		
//		S4qEInterfaceType s4qInterfaceTypeFromInput = interfaceTypeExporter.importInterfaceType(xml);
//		
//		EInterfaceTypeImport interfaceTypeImport = new EInterfaceTypeImportHibImpl();
//		
//		interfaceTypeImport.importData(s4qInterfaceTypeFromInput, true);

		//assertInterfaceTypeTables("test/flatFile/InterfaceType/Test1OWTrue.xml");
		
//	}
	
	
	private void assertInterfaceTypeTables(final String flatFileNamePath){
		Connection jdbcConnection = null;
		try{
		// Fetch database data after executing your code
		Class driverClass = Class.forName("org.hsqldb.jdbcDriver");
		jdbcConnection = DriverManager.getConnection(
				"jdbc:hsqldb:mem:/spagobi","sa","");
		IDatabaseConnection connection = new DatabaseConnection(jdbcConnection);
		DatabaseConfig config = connection.getConfig();
		config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY,
				new HsqldbDataTypeFactory());
        IDataSet databaseDataSet = connection.createDataSet();
        

        // Load expected data from an XML dataset
        IDataSet expectedDataSet = new FlatXmlDataSet(new File(flatFileNamePath));
        ITable actualInterfaceTypeTable = databaseDataSet.getTable("S4Q_E_INTERFACE_TYPE");

        ITable expectedInterfaceTypeTable = expectedDataSet.getTable("S4Q_E_INTERFACE_TYPE");

        ITable actualInterfaceField = databaseDataSet.getTable("S4Q_E_INTERFACE_FIELD");

        ITable expectedInterfaceField = expectedDataSet.getTable("S4Q_E_INTERFACE_FIELD");

        ITable actualDomainValue = databaseDataSet.getTable("S4Q_E_DOMAIN_VALUE");
        
        ITable expectedDomainValue = expectedDataSet.getTable("S4Q_E_DOMAIN_VALUE");
        
        //        Assert actual database table match expected table
        Assertion.assertEquals(actualInterfaceTypeTable, expectedInterfaceTypeTable);
        
        Assertion.assertEquals(actualInterfaceField, expectedInterfaceField);
        
        Assertion.assertEquals(actualDomainValue, expectedDomainValue);
		}catch(Exception e){
			fail();
		}finally{
			if (jdbcConnection != null){
				try {
					jdbcConnection.close();
				} catch (SQLException e) {
					fail();
				}
			}
		}
		
	}
	
	private void print(){
		Connection jdbcConnection = null;
		try {
			Class driverClass = Class.forName("org.hsqldb.jdbcDriver");
		jdbcConnection = DriverManager.getConnection(
				"jdbc:hsqldb:mem:/spagobi","sa","");
		IDatabaseConnection connection = new DatabaseConnection(jdbcConnection);
		DatabaseConfig config = connection.getConfig();
		config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY,
				new HsqldbDataTypeFactory());
        
		// dependent tables database export: export table X and all tables that
		// have a PK which is a FK on X, in the right order for insertion
		// String[] depTableNames =
		// TablesDependencyHelper.getAllDependentTables(
		// connection, "s4q_e_operation");
		String[] depTableNames = { "s4q_e_interface_type",
				"s4q_e_interface_field", "s4q_e_domain_value"};

		IDataSet depDataset = connection.createDataSet(depTableNames);
		FlatXmlDataSet.write(depDataset, new FileOutputStream(
				"test/tipotapo.xml"));
		} catch (Exception e) {
			fail();
		}
	}
}
